/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.tutorial ;

import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.jdom.Element;

/**
* @author Robert Bakic
*/
public class Price
{
	private double rate = 20.0 ;	

	@DependencyInjectionInfo(desc="The method used to publish to a bean output", param={Object.class, String.class, Object.class})
	public IMethodProxy<Void> publish ;
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization.")
    public void init(Element para)
	throws Exception
	{
		// Parameter format must be checked to prevent incorrect modification of the XML file
		if(para==null)
		{   Log.severe("Wrong parameters 'para'") ;
			return ; }
		
		Element row = para.getChild("row") ;
		if(row==null)
		{   Log.severe("Wrong parameters 'row'") ;
			return ; }
		
		// Rate value
		String strate = row.getAttributeValue("valu") ; 
		if(strate!=null && !strate.isEmpty())
			rate = Double.parseDouble(strate) ;
    }
	
	@Export
	@MethodInfo(desc="This method calculate a discount")
	final public Double calculateDiscount(@ArgInfo(name="price", desc="The price before discount") Double price)
	throws Exception
	{
		Double discount = new Double(price.doubleValue()*rate/100) ;

		if(publish==null) 
			Log.warning("Missing publish method injection in Price bean") ;
		else
			publish.call(this, "PriceCalled", "Method calculateDiscount() called for price : "+price) ;
		
		return discount ;
	}
}